#include <Wire.h>
#include "Adafruit_NeoPixel.h"
#include <TimerOne.h>

#include <SoftwareSerial.h>
#define BUFFER_LENGTH 32
#define MPU9250_ADDRESS 0x68
#define MAG_ADDRESS 0x0C

#define GYRO_FULL_SCALE_250_DPS 0x00
#define GYRO_FULL_SCALE_500_DPS 0x08
#define GYRO_FULL_SCALE_1000_DPS 0x10
#define GYRO_FULL_SCALE_2000_DPS 0x18

#define ACC_FULL_SCALE_2_G 0x00
#define ACC_FULL_SCALE_4_G 0x08
#define ACC_FULL_SCALE_8_G 0x10
#define ACC_FULL_SCALE_16_G 0x18
SoftwareSerial ble(10,11);
char buffer[BUFFER_LENGTH]; // Buffer to store response
int timeout = 800;    // Wait 800ms each time for BLE to response, depending on your application, adjust this value accordingly
long bauds[] = {9600, 115200, 57600, 38400, 2400, 4800, 19200}; // common baud rates, when using HM-10 module with SoftwareSerial, try not to go over 57600

long BLEAutoBaud() {
  int baudcount=sizeof(bauds)/sizeof(long);
  for(int i=0; i<baudcount; i++) {
    for(int x=0; x<3; x++) {  // test at least 3 times for each baud
      Serial.print("Testing baud ");
      Serial.println(bauds[i]);
      ble.begin(bauds[i]);
      if (BLEIsReady()) {
        return bauds[i];
      }
    }
  }
  return -1;
}
      
boolean BLEIsReady() {
  BLECmd(timeout, "AT" ,buffer);    // Send AT and store response to buffer 
  if (strcmp(buffer,"OK")==0){    
    return true;
  } else {
    return false;
  }  
}

boolean BLECmd(long timeout, char* command, char* temp) {
  long endtime;
  boolean found=false;
  endtime=millis()+timeout;   // 
  memset(temp,0,BUFFER_LENGTH);   // clear buffer
  found=true;
  Serial.print("Arduino send = ");
  Serial.println(command);
  ble.print(command);
  
  // The loop below wait till either a response is received or timeout
  // The problem with this BLE Shield is the HM-10 module does not response with CR LF at the end of the response,
  // so a timeout is required to detect end of response and also prevent the loop locking up.

  while(!ble.available()){
    if(millis()>endtime) {      // timeout, break
      found=false;
      break;
    }
  }  

  if (found) {          // response is available
    int i=0;
    while(ble.available()) {    // loop and read the data
      char a=ble.read();
      // Serial.print((char)a); // Uncomment this to see raw data from BLE
      temp[i]=a;      // save data to buffer
      i++;
      if (i >= BUFFER_LENGTH) break;  // prevent buffer overflow, need to break
      delay(1);     // give it a 2ms delay before reading next character
    }
    Serial.print("BLE reply    = ");
    Serial.println(temp);
    return true;
  } else {
    Serial.println("BLE timeout!");
    return false;
  }
}
// This function read Nbytes bytes from I2C device at address Address.
// Put read bytes starting at register Register in the Data array.
void I2Cread(uint8_t Address, uint8_t Register, uint8_t Nbytes, uint8_t* Data)
{
  // Set register address
  Wire.beginTransmission(Address);
  Wire.write(Register);
  Wire.endTransmission();

  // Read Nbytes
  Wire.requestFrom(Address, Nbytes);
  uint8_t index = 0;
  while (Wire.available())
    Data[index++] = Wire.read();
}


// Write a byte (Data) in device (Address) at register (Register)
void I2CwriteByte(uint8_t Address, uint8_t Register, uint8_t Data)
{
  // Set register address
  Wire.beginTransmission(Address);
  Wire.write(Register);
  Wire.write(Data);
  Wire.endTransmission();
}

// Initial time
long int ti;
volatile bool intFlag = false;

// ,      .
#define LED_COUNT 55

// ,       DIN.
#define LED_PIN 9

//   strip    .
Adafruit_NeoPixel strip = Adafruit_NeoPixel(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

float stp(float x, float v)
{
  if (x > v) return (v);
  if ((x > -v) && (x < v)) return (x);
  if (x < (-v)) return (-v);
}

void HSV_to_RGB(float h, float s, float v, byte *r, byte *g, byte *b)
{
  int i, f, p, q, t;

  h = max(0.0, min(360.0, h));
  s = max(0.0, min(100.0, s));
  v = max(0.0, min(100.0, v));

  s /= 100;
  v /= 100;

  if (s == 0) {
    // Achromatic (grey)
    *r = *g = *b = round(v * 255);
    return;
  }

  h /= 60; // sector 0 to 5
  i = floor(h);
  f = h - i; // factorial part of h
  p = v * (1 - s);
  q = v * (1 - s * f);
  t = v * (1 - s * (1 - f));
  switch (i) {
    case 0:
      *r = round(255 * v);
      *g = round(255 * t);
      *b = round(255 * p);
      break;
    case 1:
      *r = round(255 * q);
      *g = round(255 * v);
      *b = round(255 * p);
      break;
    case 2:
      *r = round(255 * p);
      *g = round(255 * v);
      *b = round(255 * t);
      break;
    case 3:
      *r = round(255 * p);
      *g = round(255 * q);
      *b = round(255 * v);
      break;
    case 4:
      *r = round(255 * t);
      *g = round(255 * p);
      *b = round(255 * v);
      break;
    default: // case 5:
      *r = round(255 * v);
      *g = round(255 * p);
      *b = round(255 * q);
  }
}
float f(float x)
{
  float k;
  k = (stp(x, 90) + 90) * 16 / 180;
  return k;
}
float y, dt, cy, wy;
int n = 0;
void setup() {
  Wire.begin();
  Serial.begin(9600);

  // Set accelerometers low pass filter at 5Hz
  I2CwriteByte(MPU9250_ADDRESS, 29, 0x06);
  // Set gyroscope low pass filter at 5Hz
  I2CwriteByte(MPU9250_ADDRESS, 26, 0x06);


  // Configure gyroscope range
  I2CwriteByte(MPU9250_ADDRESS, 27, GYRO_FULL_SCALE_1000_DPS);
  // Configure accelerometers range
  I2CwriteByte(MPU9250_ADDRESS, 28, ACC_FULL_SCALE_4_G);
  // Set by pass mode for the magnetometers
  I2CwriteByte(MPU9250_ADDRESS, 0x37, 0x02);

  // Request continuous magnetometer measurements in 16 bits
  I2CwriteByte(MAG_ADDRESS, 0x0A, 0x16);

  pinMode(13, OUTPUT);
  Timer1.initialize(10000); // initialize timer1, and set a 1/2 second period
  Timer1.attachInterrupt(callback); // attaches callback() as a timer overflow interrupt


  // Store initial time
  ti = millis();
  strip.begin();
  strip.setBrightness(80);
  pinMode(13, OUTPUT);
  digitalWrite(13,LOW);
pinMode(50,OUTPUT);
digitalWrite(50,HIGH);
pinMode(40,OUTPUT);
digitalWrite(40,HIGH);
  // If you see lots of BLE timeout on Serial Monitor after BLEAutoBaud completed, most likely your have a bad shield
  // Check if the shield JUMPER is correctly set to 
  // HM10 TX to D2
  // HM10 RX to D3
  
  long baudrate = BLEAutoBaud();

  if (baudrate>0) {
    Serial.print("Found BLE baud rate ");
    Serial.println(baudrate);
  } else {
    Serial.println("No BLE detected.");
    ble.begin(9600);     // No BLE found, just going to stop here
  }

  // The following commands are just to demonstrate the shield is working properly,
  // in actual application, only call those that are needed by your application.
  // Check HM-10 datasheet for the description of the commands.
  BLECmd(timeout,"AT+NAME?",buffer);
  BLECmd(timeout,"AT+BAUD?",buffer);
  BLECmd(timeout,"AT+MODE?",buffer);
  BLECmd(timeout,"AT+PASS?",buffer);
  BLECmd(timeout,"AT+VERS?",buffer);
  BLECmd(timeout,"AT+RADD?",buffer);
  BLECmd(timeout,"AT+ADDR?",buffer);
  BLECmd(timeout,"AT+TYPE?",buffer);
  BLECmd(timeout,"AT+POWE?",buffer);
  BLECmd(timeout,"AT+NOTI1",buffer);
  Serial.println("----------------------");
  Serial.println("Waiting for remote connection...");
while(!ble.available()) { Serial.println("..."); delay(1000);}
  while (ble.available()) {
    char c =(char)ble.read();
    Serial.print(c);
  } 
  
   cy = 0;
  n = 0;
  uint8_t Buf[14];
  while ((millis() - ti) < 500)
  {
    I2Cread(MPU9250_ADDRESS, 0x3B, 14, Buf);
    delay(5);
  }
   Serial.println("Calib");
  int16_t ti = millis();
  while ((millis() - ti) < 8000)
  {
    I2Cread(MPU9250_ADDRESS, 0x3B, 14, Buf);
    int16_t gy = (Buf[10] << 8 | Buf[11]);
    Serial.println(gy);
    n++;
    cy += float(gy);
  }
  cy = cy / float(n);
  Serial.println(cy);
  y = 0;
}
 
// Counter
long int cpt=0;
 
void callback()
{ 
intFlag=true;
digitalWrite(13, digitalRead(13) ^ 1);
}
float r = 0, g = 0, b = 0;
byte index = 0;
bool flag=false;
void loop() {
static float __t = millis();
  while (!intFlag);
  intFlag = false;

  // Display time


  // _______________
  // ::: Counter :::

  // Display data counter
  // Serial.print (cpt++,DEC);
  // Serial.print ("\t");



  // ____________________________________
  // ::: accelerometer and gyroscope :::

  // Read accelerometer and gyroscope
  uint8_t Buf[14];
  I2Cread(MPU9250_ADDRESS, 0x3B, 14, Buf);

  // Create 16 bits values from 8 bits data

  // Accelerometer
  int16_t ax = -(Buf[0] << 8 | Buf[1]);
  int16_t ay = -(Buf[2] << 8 | Buf[3]);
  int16_t az = Buf[4] << 8 | Buf[5];

  // Gyroscope
  int16_t gx = -(Buf[8] << 8 | Buf[9]);
  int16_t gy = -(Buf[10] << 8 | Buf[11]);
  int16_t gz = Buf[12] << 8 | Buf[13];

  // Display values

  // Accelerometer
  /*Serial.print (ax,DEC);
    Serial.print ("\t");
    Serial.print (ay,DEC);
    Serial.print ("\t");
    Serial.print (az,DEC);
    Serial.print ("\t");

    // Gyroscope
    Serial.print (gx,DEC);
    Serial.print ("\t");
    Serial.print (gy,DEC);
    Serial.print ("\t");
    Serial.print (gz,DEC);
    Serial.print ("\t");

  */
  // _____________________
  // ::: Magnetometer :::


  // Read register Status 1 and wait for the DRDY: Data Ready

  uint8_t ST1;
  do
  {
    I2Cread(MAG_ADDRESS, 0x02, 1, &ST1);
  }
  while (!(ST1 & 0x01));

  // Read magnetometer data
  uint8_t Mag[7];
  I2Cread(MAG_ADDRESS, 0x03, 7, Mag);

  // Create 16 bits values from 8 bits data

  // Magnetometer
  int16_t mx = -(Mag[3] << 8 | Mag[2]);
  int16_t my = -(Mag[1] << 8 | Mag[0]);
  int16_t mz = -(Mag[5] << 8 | Mag[4]);


  // Magnetometer
 
  //Serial.print ("\t");
  //Serial.print (my-70,DEC);
  //Serial.print ("\t");
  //Serial.print (mz-700,DEC);
  //Serial.print ("\t");



  // End of line
  Serial.println("");
  // delay(100);

  
 /* Serial.print("AngleX:");
    Serial.println(mx);
    Serial.print("AngleY:");
      Serial.println(my);
      Serial.print("AngleZ:");
        Serial.println(mz);*/
  //   
  int _r[8] = {255, 255, 255, 0,    0,    255, 0, 255};
  int _g[8] = {255, 0,   255, 0,    255, 0,   255, 0};
  int _b[8] = {0,   255, 255, 255,  255, 0,   0, 0}; 
  r = _r[int(round((abs(mx) % 180) / 90) + round((abs(my) % 180) / 90) * 2 + round((abs(mz) % 180) / 90) * 4)] * 0.07 + r * 0.93;
  g = _g[int(round((abs(mx) % 180) / 90) + round((abs(my) % 180) / 90) * 2 + round((abs(mz) % 180) / 90) * 4)] * 0.07 + g * 0.93;
  b = _b[int(round((abs(mx) % 180) / 90) + round((abs(my) % 180) / 90) * 2 + round((abs(mz) % 180) / 90) * 4)] * 0.07 + b * 0.93;
  for (int i = 0; i < LED_COUNT; i++)
  {
   // Serial.println(r);
    strip.setPixelColor(i, strip.Color(r,g,b)); //  .
  }
  //   .
  strip.show();
  //  100 
wy = float(gy) + float(cy);
  dt = (millis() - __t) / 1000;
  __t = millis();
  y += (wy * dt) / 32;
while (y < 0) y += 360;
y -= 180;
  index += 1;
  if (index > 205) index = 0;
//if (digitalRead(41)=HIGH) 
  ble.write(int8_t(round(y/3)));
  delay(10);
}